import { Injectable, InjectionToken, Component, Inject, HostListener, Injector, ɵɵdefineInjectable, ɵɵinject, INJECTOR, NgModule } from '@angular/core';
import { Overlay, OverlayModule } from '@angular/cdk/overlay';
import { ComponentPortal, PortalInjector } from '@angular/cdk/portal';
import { timer, BehaviorSubject, defer, interval } from 'rxjs';
import { take, withLatestFrom, filter, map, share, reduce } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NotificationData {
}
class NotificationAlertContainerStyles {
}
class DefaultNotificationConfig {
    constructor() {
        this.animation = true;
        this.dismissible = true;
        this.position = 'topLeft';
        this.timeOut = 5;
        this.unique = false;
    }
}
DefaultNotificationConfig.decorators = [
    { type: Injectable }
];
/** @type {?} */
const defaultNotificationCofigValues = {
    position: 'topRight',
    animation: true,
    timeOut: 5,
    dismissible: true
};
/** @type {?} */
const NOTIFICATION_CONFIG_TOKEN = new InjectionToken('notification-config');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxNotificationRef {
    /**
     * @param {?} overlay
     * @param {?=} nt
     */
    constructor(overlay, nt) {
        this.overlay = overlay;
        this.nt = nt;
    }
    /**
     * @return {?}
     */
    close() {
        this.overlay.dispose();
        this.clearMessage();
    }
    /**
     * @return {?}
     */
    isVisible() {
        return this.overlay && this.overlay.overlayElement;
    }
    /**
     * @return {?}
     */
    getPosition() {
        return this.overlay.overlayElement.getBoundingClientRect();
    }
    /**
     * @return {?}
     */
    clearMessage() {
        this.nt.lastMessage = null;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxBootstrapAlertNotificationComponent {
    /**
     * @param {?} data
     * @param {?} ref
     * @param {?} containerStyles
     * @param {?} notificationConfig
     */
    constructor(data, ref, containerStyles, notificationConfig) {
        this.data = data;
        this.ref = ref;
        this.containerStyles = containerStyles;
        this.notificationConfig = notificationConfig;
        this.animatedClass = 'fadeInDown';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.notificationConfig.dismissible) {
            this.dismissibleTimer();
        }
        this.show = true;
    }
    /**
     * @return {?}
     */
    close() {
        if (this.notificationConfig.animation) {
            this.animatedClass = 'fadeOutUp';
            this.show = false;
            timer(1000)
                .pipe(take(1))
                .subscribe((/**
             * @param {?} t
             * @return {?}
             */
            t => this.ref.close()));
        }
        else {
            this.show = false;
            this.ref.close();
        }
    }
    /**
     * @return {?}
     */
    onMouseOver() {
        if (this.notificationConfig.dismissible) {
            this.notification.paused.next(true);
        }
    }
    /**
     * @return {?}
     */
    onMouseLeave() {
        if (this.notificationConfig.dismissible) {
            this.notification.paused.next(false);
        }
    }
    /**
     * @private
     * @return {?}
     */
    dismissibleTimer() {
        /** @type {?} */
        const subject = new BehaviorSubject(false);
        this.notification = {
            paused: subject,
            obs: this.getPausableTimer(this.notificationConfig.timeOut, subject)
        };
        this.notification.obs.completeTimer
            .pipe(take(1))
            .subscribe((/**
         * @param {?} t
         * @return {?}
         */
        t => this.close()));
    }
    /**
     * @private
     * @param {?} timeout
     * @param {?} pause
     * @return {?}
     */
    getPausableTimer(timeout, pause) {
        /** @type {?} */
        const pausableTimer$ = defer((/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            let seconds = 1;
            return interval(1000).pipe(withLatestFrom(pause), filter((/**
             * @param {?} __0
             * @return {?}
             */
            ([v, paused]) => !paused)), take(timeout), map((/**
             * @return {?}
             */
            () => seconds++)));
        })).pipe(share());
        return {
            stepTimer: pausableTimer$,
            completeTimer: pausableTimer$.pipe(reduce((/**
             * @param {?} x
             * @param {?} y
             * @return {?}
             */
            (x, y) => y)))
        };
    }
}
NgxBootstrapAlertNotificationComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-bootstrap-alert-notification',
                template: `
    <div [ngStyle]="containerStyles" class="col-11 col-sm-4 notification-container">
      <div *ngIf="!data.template" [class.alert-dismissible]="notificationConfig.dismissible"
           [class.alert-with-icon]="data.icon" [class.animated]="notificationConfig.animation"
           [class.show]="show"
           [ngClass]="['alert-'+data.type, animatedClass]" class="alert fade" role="alert">
        <button (click)="close()" aria-label="Close" class="close" type="button">
          <span aria-hidden="true">×</span>
        </button>
        <h4 *ngIf="data.title as title" class="title">{{title}}</h4>
        <span *ngIf="data.icon as icon" [className]="icon" data-notify="icon"></span>
        <span data-notify="message">
      {{data.message}}
    </span>
      </div>
      <ng-container *ngIf="data.template as templateRef" [ngTemplateOutlet]="templateRef"></ng-container>
    </div>

  `,
                styles: [`
      :host {
        z-index: 1051;
        position: fixed;
        display: block;
      }

      .title {
        margin-bottom: 0;
        color: white;
      }

      @keyframes fadeInDown {
        from {
          opacity: 0;
          transform: translate3d(0, -100%, 0);
        }

        to {
          opacity: 1;
          transform: translate3d(0, 0, 0);
        }
      }

      .fadeInDown {
        animation-name: fadeInDown;
      }

      @keyframes fadeOutUp {
        from {
          opacity: 1;
        }

        to {
          opacity: 0;
          transform: translate3d(0, -100%, 0);
        }
      }

      .fadeOutUp {
        animation-name: fadeOutUp;
      }

      .animated {
        animation-duration: 1s;
        animation-fill-mode: both;
      }
    `]
            }] }
];
/** @nocollapse */
NgxBootstrapAlertNotificationComponent.ctorParameters = () => [
    { type: NotificationData },
    { type: NgxNotificationRef },
    { type: NotificationAlertContainerStyles },
    { type: undefined, decorators: [{ type: Inject, args: [DefaultNotificationConfig,] }] }
];
NgxBootstrapAlertNotificationComponent.propDecorators = {
    onMouseOver: [{ type: HostListener, args: ['mouseover',] }],
    onMouseLeave: [{ type: HostListener, args: ['mouseleave',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const Styles = {
    display: 'inline-block',
    margin: '0px auto',
    position: 'fixed',
    transition: 'all 0.5s ease-in-out',
    zIndex: '1031'
};
class NgxBootstrapAlertNotificationService {
    /**
     * @param {?} overlay
     * @param {?} parentInjector
     * @param {?} notificationConfig
     */
    constructor(overlay, parentInjector, notificationConfig) {
        this.overlay = overlay;
        this.parentInjector = parentInjector;
        this.notificationConfig = notificationConfig;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set lastMessage(value) {
        this._lastMessage = value;
    }
    /**
     * @param {?} data
     * @param {?=} config
     * @return {?}
     */
    show(data, config) {
        if (!config || config && !config.unique ||
            (config && config.unique &&
                ((Array.isArray(this._lastMessage) &&
                    (this._lastMessage || []).includes(data.message)) ||
                    this._lastMessage !== data.message))) {
            this._lastMessage = data.message;
            /** @type {?} */
            const positionStrategy = this.getPositionStrategy();
            /** @type {?} */
            const overlayRef = this.overlay.create({ positionStrategy });
            /** @type {?} */
            const notificationRef = new NgxNotificationRef(overlayRef, this.parentInjector.get(NgxBootstrapAlertNotificationService));
            /** @type {?} */
            const injector = this.getInjector(data, notificationRef, this.parentInjector, config, this.createContainerStyles(config));
            this.lastNotification = notificationRef;
            /** @type {?} */
            const toastPortal = new ComponentPortal(NgxBootstrapAlertNotificationComponent, null, injector);
            overlayRef.attach(toastPortal);
            return notificationRef;
        }
        return null;
    }
    /**
     * @return {?}
     */
    getPositionStrategy() {
        return this.overlay.position().global();
    }
    /**
     * @param {?} data
     * @param {?} notificationRef
     * @param {?} parentInjector
     * @param {?=} notificationConfig
     * @param {?=} containerStyles
     * @return {?}
     */
    getInjector(data, notificationRef, parentInjector, notificationConfig, containerStyles) {
        /** @type {?} */
        const tokens = new WeakMap();
        tokens.set(NotificationData, data);
        tokens.set(NgxNotificationRef, notificationRef);
        if (notificationConfig) {
            notificationConfig = Object.assign({}, this.notificationConfig, notificationConfig);
            tokens.set(DefaultNotificationConfig, notificationConfig);
        }
        if (containerStyles) {
            tokens.set(NotificationAlertContainerStyles, containerStyles);
        }
        return new PortalInjector(parentInjector, tokens);
    }
    /**
     * @private
     * @param {?=} config
     * @return {?}
     */
    createContainerStyles(config) {
        /** @type {?} */
        const styles = Object.assign({}, Styles);
        /** @type {?} */
        let notificationContainer;
        if (this.lastNotification) {
            /** @type {?} */
            const element = this.lastNotification.isVisible();
            notificationContainer = element
                ? element.querySelector('.notification-container')
                : null;
        }
        /** @type {?} */
        const conf = Object.assign({}, this.notificationConfig, (config || {}));
        if (conf.position && conf.position.includes('top')) {
            styles.top =
                notificationContainer && notificationContainer.style.top
                    ? `${parseFloat(notificationContainer.style.top) +
                        notificationContainer.clientHeight}px`
                    : '20px';
            switch (conf.position) {
                case 'topLeft':
                    styles.left = '20px';
                    break;
                case 'topCenter':
                    styles.left = '0px';
                    styles.right = '0px';
                    break;
                case 'topRight':
                    styles.right = '20px';
                    break;
                default:
                    break;
            }
        }
        else {
            styles.bottom =
                notificationContainer && notificationContainer.style.bottom
                    ? `${parseFloat(notificationContainer.style.bottom) +
                        notificationContainer.clientHeight}px`
                    : '20px';
            switch (conf.position) {
                case 'bottomLeft':
                    styles.left = '20px';
                    break;
                case 'bottomCenter':
                    styles.left = '0px';
                    styles.right = '0px';
                    break;
                case 'bottomRight':
                    styles.right = '20px';
                    break;
                default:
                    break;
            }
        }
        return styles;
    }
}
NgxBootstrapAlertNotificationService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
NgxBootstrapAlertNotificationService.ctorParameters = () => [
    { type: Overlay },
    { type: Injector },
    { type: undefined, decorators: [{ type: Inject, args: [DefaultNotificationConfig,] }] }
];
/** @nocollapse */ NgxBootstrapAlertNotificationService.ngInjectableDef = ɵɵdefineInjectable({ factory: function NgxBootstrapAlertNotificationService_Factory() { return new NgxBootstrapAlertNotificationService(ɵɵinject(Overlay), ɵɵinject(INJECTOR), ɵɵinject(DefaultNotificationConfig)); }, token: NgxBootstrapAlertNotificationService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxBootstrapAlertNotificationModule {
    /**
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: NgxBootstrapAlertNotificationModule,
            providers: [
                {
                    provide: DefaultNotificationConfig,
                    useValue: config ? config : defaultNotificationCofigValues
                }
            ]
        };
    }
}
NgxBootstrapAlertNotificationModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NgxBootstrapAlertNotificationComponent],
                imports: [CommonModule, OverlayModule],
                entryComponents: [NgxBootstrapAlertNotificationComponent],
                exports: [NgxBootstrapAlertNotificationComponent]
            },] }
];

export { NgxBootstrapAlertNotificationComponent, NgxBootstrapAlertNotificationModule, NgxBootstrapAlertNotificationService, NotificationData as ɵa, NotificationAlertContainerStyles as ɵb, DefaultNotificationConfig as ɵd, defaultNotificationCofigValues as ɵe, NgxNotificationRef as ɵf };
//# sourceMappingURL=benevideschissanga-ngx-bootstrap-alert-notification.js.map
